;;; rsync://user:pass@server:port/path/file
;;; rsync://user:pass@server:rsh/path/file
;;; rsync://user:pass@server:ssh/path/file
;;; rcp://user:pass@server/path/file
;;; scp://user:pass@server/path/file

;;; accessors for mirror abstract object
(defun murl-type (mirror) (elt mirror 0))
(defun murl-user (mirror) (elt mirror 1))
(defun murl-pass (mirror) (elt mirror 2))
(defun murl-host (mirror) (elt mirror 3))
(defun murl-port (mirror) (elt mirror 4))
(defun murl-path (mirror) (elt mirror 5))


(defun murl-canonical-host (murl)
  (let ((host (murl-host murl))
	(user (murl-user murl)))
    (if user (format "%s@%s" user host) host)))

(defun murl-canonical (murl)
  (let ((type (murl-type murl))
	(path (murl-path murl)))
    (cond ((eq type 'rsync)
	   (let* ((port (murl-port murl))
		  (rsh (and (not (null port)) (symbolp port))))
	     (concat (murl-canonical-host murl)
		     (if rsh ":" "::")
		     (if rsh path (substring path 1)))))
	  ((memq type '(scp rcp))
	   (format "%s:%s"
		   (murl-canonical-host murl)
		   path))
	  (t (concat (murl-canonical-host murl)
		     path))
	  )))


(defvar mirror-id-regexp "[^;?@:/]*")
(defvar mirror-type-regexp "\\(rsync\\|rcp\\|scp\\)://")
(defvar mirror-pass-regexp (concat mirror-id-regexp "@"))
(defvar mirror-user-only-regexp (concat mirror-id-regexp "\\(:\\|@\\)"))
(defvar mirror-user-regexp (concat mirror-id-regexp
				   "\\(:" mirror-pass-regexp "\\|@\\)"))
(defvar mirror-host-regexp mirror-id-regexp)
(defvar mirror-port-regexp ":\\([0-9]+\\|rsh\\|ssh\\)")
(defvar mirror-path-regexp (concat "\\(" "/" mirror-id-regexp "\\)*"))

(defvar mirror-regexp
  (concat mirror-type-regexp
	  "\\(" mirror-user-regexp "\\)?"
	  "\\(" mirror-pass-regexp "\\)?"
	  "\\(" mirror-host-regexp "\\)?"
	  "\\(" mirror-port-regexp "\\)?"
	  mirror-path-regexp))

(defvar mirror-parser-list
  (list (cons mirror-type-regexp (lambda (s) (read (substring s 0 (- (length s) 3)))))
	(cons mirror-user-only-regexp (lambda (s) (substring s 0 (- (length s) 1))))
	(cons mirror-pass-regexp (lambda (s) (substring s 0 (- (length s) 1))))
	(cons mirror-host-regexp (lambda (s) (and (< 0 (length s)) s)))
	(cons mirror-port-regexp (lambda (s) (read (substring s 1))))
	(cons mirror-path-regexp (lambda (s) s))
	))

(defun mirror-parse-url (url)
  (let ((parse-buffer (get-buffer-create "*mirror parse*")))
    (save-excursion
      (set-buffer parse-buffer)
      (delete-region (point-min) (point-max))
      (insert url)
      (goto-char (point-min))
      (let ((url-obj
	     (let ((loop
		     (lambda (regexs strings)
		       (if (null regexs)
			   (progn
			     ;(message "plonk: %S" strings)
			     (vconcat (reverse strings)))
			 (let* ((parser (car regexs))
				(regex (car parser))
				(canon (cdr parser))
				(remaining (cdr regexs))
				(string (and (looking-at regex)
					     (prog1 (match-string 0)
					       (goto-char (match-end 0))))))
			   (funcall loop
				    remaining
				    (cons (and string (funcall canon string))
					  strings)))
			 ))))
	       (funcall loop mirror-parser-list nil))))
	
	url-obj)
  )))

(provide 'mirror-url)